/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file drag.h
 *
 * @type subsystem
 *
 * @description
 *
 * The drag subsystem handles dragging around canvasses.
 */

#ifndef __rov_drag
#define __rov_drag


#ifdef __cplusplus
extern "C"
{
#endif

#ifndef __rov_main
#include "main.h"
#endif

/**
 * Called when the dragging has completed.
 * @param mouse_str the mouse at the end of the drag.
 */
typedef void (* drag_endhandler)(main_mousestr *mouse_str);

/**
 * Called duringa a dragging operation.
 * @param x the x coordinate in the current canvas.
 * @param y the y coordinate in the current canvas.
 */
typedef void (* drag_durhandler)(int x,
				 int y);

/**
 * Drag a canvas around the screen.
 * @param c the canvas to drag around.
 * @note This is identical to dragging the window's title bar to move
 * it around, except it can be called with the mouse anywhere on the
 * screen.
 */
void drag_canvas(canvas c);

#ifdef OLD_DRAGSTYLE
/* For definitions of the variable parameters, see the main documentation */
void drag_outline(canvas c, int w, int h, int screen, ...);
void drag_hidden(canvas c, int screen, ...);
void drag_htrench(canvas c, int, int, int, ...);
void drag_vtrench(canvas c, int, int, int, ...);
void drag_trench(canvas c, int x0, int y0, int x1, int y1, ...);
int drag_sprite(void *spr_area , char *spr_name, int screen, ...);
void drag_spritestop(void);
#else

/**
 * Drag a dotted outline around the screen or canvas.
 * @param c the canvas to originate from.
 * @param w the width of the box.
 * @param h the height of the box.
 * @param screen a flag indicating whether the screen or the canvas gives
 * the bounding limits. Non-zero means the screen gives the bounding limits.
 * @param proc the function to call when dragging is complete.
 * @note This is normally used to create outline drags for save boxes.
 * @see drag_endhandler()
 */
void drag_outline(canvas c,
		  int w,
		  int h,
		  int screen,
		  drag_endhandler proc);

/**
 * Drag a hidden point around the screen or canvas
 * @param c the canvas to originate from
 * @param screen a flag indicating whether the screen or the canvas gives
 * the bounding limits. Non-zero means the screen gives the bounding limits.
 * @param endproc the function to call when dragging is complete.
 * @param during the function to call during a drag (NULL for none).
 * @see drag_endhandler()
 * @see drag_durhandler()
 */
void drag_hidden(canvas c,
		 int screen,
		 drag_endhandler endproc,
		 drag_durhandler during);

/**
 * Drag a point in a horizontal trench
 * @param c the canvas.
 * @param x the left hand (canvas) coordinate.
 * @param y the (canvas) y coordinate of the trench.
 * @param l the length of the trench.
 * @param endproc the function to call when dragging is complete.
 * @param during the function to call during a drag (NULL for none).
 * @note this is typically used to create sliders.
 * @see drag_vtrench()
 * @see drag_trench()
 * @see drag_endhandler()
 * @see drag_durhandler()
 */
void drag_htrench(canvas c,
		  int x, 
		  int y,
		  int l,
		  drag_endhandler endproc,
		  drag_durhandler during);

/**
 * Drag a point in a vertical trench
 * @param c the canvas.
 * @param x the (canvas) x coordinate of the trench.
 * @param y the bottom (canvas) coordinate.
 * @param l the length of the trench.
 * @param endproc the function to call when dragging is complete.
 * @param during the function to call during a drag (NULL for none).
 * @note this is typically used to create sliders.
 * @see drag_htrench()
 * @see drag_trench()
 * @see drag_endhandler()
 * @see drag_durhandler()
 */
void drag_vtrench(canvas c,
		  int x, 
		  int y,
		  int l,
		  drag_endhandler endproc,
		  drag_durhandler during);

/**
 * Drag a point in a trench
 * @param c the canvas.
 * @param x0 the left hand (canvas) coordinate.
 * @param y0 the bottom (canvas) coordinate.
 * @param x1 the right hand (canvas) coordinate.
 * @param y1 the top (canvas) coordinate.
 * @param endproc the function to call when dragging is complete.
 * @param during the function to call during a drag (NULL for none).
 * @note this is typically used to create 2D controllers.
 * @see drag_htrench()
 * @see drag_vtrench()
 * @see drag_endhandler()
 * @see drag_durhandler()
 */
void drag_trench(canvas c,
		 int x0, 
		 int y0,
		 int x1,
		 int y1,
		 drag_endhandler endproc,
		 drag_durhandler during);

/**
 * Drag a sprite around the canvas or screen
 * @param spr_area the sprite area.
 * @param spr_name the sprite name.
 * @param screen a flag indicating whether the screen or the canvas gives
 * the bounding limits. Non-zero means the screen gives the bounding limits.
 * @param endproc the function to call when dragging is complete.
 * @return a success state (0 for failure, non-zero for success).
 */
int drag_sprite(void *spr_area,
		char *spr_name,
		int screen,
		drag_endhandler endproc);

/**
 * Stop dragging a sprite
 */
void drag_spritestop(void);

#endif

#define DRAG_SCREEN	(1)
#define DRAG_CANVAS	(0)


#ifdef __cplusplus
}
#endif

#endif
